/*
 *  myglu.h
 *  Checkerboard
 *
 *  Created by vorg on 2009-03-12.
 *  Copyright 2009 Vorg, http://www.vorg.pl. All rights reserved.
 *
 */

#import <QuartzCore/QuartzCore.h>

// t: current time, b: beginning value, c: change in value, d: duration
float bounceEaseOut(float t, float b, float c, float d) {
	if ((t/=d) < (1/2.75)) {
		return c*(7.5625*t*t) + b;
	} else if (t < (2/2.75)) {
		return c*(7.5625*(t-=(1.5/2.75))*t + .75) + b;
	} else if (t < (2.5/2.75)) {
		return c*(7.5625*(t-=(2.25/2.75))*t + .9375) + b;
	} else {
		return c*(7.5625*(t-=(2.625/2.75))*t + .984375) + b;
	}
}

GLuint loadTexture(NSString* imageFile) 
{
	GLuint textureId;
	CGImageRef spriteImage;
	CGContextRef spriteContext;
	GLubyte *spriteData;
	size_t	width, height;
	
	spriteImage = [UIImage imageNamed:imageFile].CGImage;
	// Get the width and height of the image
	width = CGImageGetWidth(spriteImage);
	height = CGImageGetHeight(spriteImage);
	// Texture dimensions must be a power of 2. If you write an application that allows users to supply an image,
	// you'll want to add code that checks the dimensions and takes appropriate action if they are not a power of 2.
	
	if(spriteImage) {
		// Allocated memory needed for the bitmap context
		spriteData = (GLubyte *) malloc(width * height * 4);
		// Uses the bitmatp creation function provided by the Core Graphics framework. 
		spriteContext = CGBitmapContextCreate(spriteData, width, height, 8, width * 4, CGImageGetColorSpace(spriteImage), kCGImageAlphaPremultipliedLast);
		// After you create the context, you can draw the sprite image to the context.
		CGContextDrawImage(spriteContext, CGRectMake(0.0, 0.0, (CGFloat)width, (CGFloat)height), spriteImage);
		// You don't need the context at this point, so you need to release it to avoid memory leaks.
		CGContextRelease(spriteContext);
		
		// Use OpenGL ES to generate a name for the texture.
		glGenTextures(1, &textureId);
		// Bind the texture name. 
		glBindTexture(GL_TEXTURE_2D, textureId);
		// Speidfy a 2D texture image, provideing the a pointer to the image data in memory
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, spriteData);
		// Release the image data
		free(spriteData);
		
		// Set the texture parameters to use a minifying filter and a linear filer (weighted average)
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		
		// Enable use of the texture
		glEnable(GL_TEXTURE_2D);		
	}		
	
	return textureId;
}

void perspective(double fovy, double aspect, double zNear, double zFar)
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	
	double xmin, xmax, ymin, ymax;
	
	ymax = zNear * tan(fovy * M_PI / 360.0);
	ymin = -ymax;
	xmin = ymin * aspect;
	xmax = ymax * aspect;
	
	
	glFrustumf(xmin, xmax, ymin, ymax, zNear, zFar);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);		
	glDepthMask(GL_TRUE);
}

void startOrtho()		   
{               
    GLint width = 320;
	GLint height = 480;
	glMatrixMode(GL_PROJECTION);	
	glPushMatrix();
	glLoadIdentity();					   
	glOrthof( -width / 2, width / 2, -height / 2, height / 2, -1, 1 );
	glMatrixMode(GL_MODELVIEW);				
	glPushMatrix();	
	glLoadIdentity();
	glTranslatef(160.0f, 240.0f, 0.0f );
	glRotatef(270.0, 0.0, 0.0, 1.0);
	glScalef(1.0, -1.0, 1.0);					
	glDepthMask(GL_FALSE);
	glDisable(GL_DEPTH_TEST);
}

void endOrtho() {
	glMatrixMode(GL_PROJECTION);	
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);				
	glPopMatrix();	
	glDepthMask(GL_TRUE);
	glEnable(GL_DEPTH_TEST);
}

void lookAt(GLfloat eyex, GLfloat eyey, GLfloat eyez,			   
			   GLfloat centerx, GLfloat centery, GLfloat centerz,			   
			   GLfloat upx, GLfloat upy, GLfloat upz)

{	
	glMatrixMode(GL_MODELVIEW);	
    GLfloat m[16];	
    GLfloat x[3], y[3], z[3];	
    GLfloat mag;
	
    /* Make rotation matrix */
	
    /* Z vector */
    z[0] = eyex - centerx;
    z[1] = eyey - centery;
    z[2] = eyez - centerz;
    mag = sqrt(z[0] * z[0] + z[1] * z[1] + z[2] * z[2]);
    if (mag) {                   /* mpichler, 19950515 */
		z[0] /= mag;
		z[1] /= mag;
		z[2] /= mag;
    }
	
    /* Y vector */
    y[0] = upx;	
    y[1] = upy;	
    y[2] = upz;
	
    /* X vector = Y cross Z */
    x[0] = y[1] * z[2] - y[2] * z[1];
    x[1] = -y[0] * z[2] + y[2] * z[0];
    x[2] = y[0] * z[1] - y[1] * z[0];
	
    /* Recompute Y = Z cross X */	
    y[0] = z[1] * x[2] - z[2] * x[1];	
    y[1] = -z[0] * x[2] + z[2] * x[0];	
    y[2] = z[0] * x[1] - z[1] * x[0];	
		
    /* mpichler, 19950515 */	
    /* cross product gives area of parallelogram, which is < 1.0 for	 
     * non-perpendicular unit-length vectors; so normalize x, y here	 
     */
	
    mag = sqrt(x[0] * x[0] + x[1] * x[1] + x[2] * x[2]);
	
    if (mag) {		
		x[0] /= mag;		
		x[1] /= mag;		
		x[2] /= mag;		
    }
	
    mag = sqrt(y[0] * y[0] + y[1] * y[1] + y[2] * y[2]);	
    if (mag) {		
		y[0] /= mag;		
		y[1] /= mag;		
		y[2] /= mag;		
    }
	
#define M(row,col)  m[col*4+row]	
    M(0, 0) = x[0];	
    M(0, 1) = x[1];	
    M(0, 2) = x[2];	
    M(0, 3) = 0.0;	
    M(1, 0) = y[0];	
    M(1, 1) = y[1];	
    M(1, 2) = y[2];	
    M(1, 3) = 0.0;	
    M(2, 0) = z[0];	
    M(2, 1) = z[1];	
    M(2, 2) = z[2];	
    M(2, 3) = 0.0;	
    M(3, 0) = 0.0;	
    M(3, 1) = 0.0;	
    M(3, 2) = 0.0;	
    M(3, 3) = 1.0;	
#undef M
	
    glMultMatrixf(m);
    /* Translate Eye to Origin */
	
    glTranslatef(-eyex, -eyey, -eyez);
}
